#[non_exhaustive]
pub enum ContiguousMemoryError { NoStorageLeft, AlreadyUsed, NotContained, DoubleFree, Unshrinkable { required_size: usize, }, Lock(LockingError), TrackerInUse, Layout(LayoutError), BorrowMut(RegionBorrowed), }
Represents errors that can occur while using the ContiguousMemoryStorage container.

Variants (Non-exhaustive)

This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.


NoStorageLeft

Tried to store data that does not fit into any of the remaining free memory regions.



AlreadyUsed

Attempted to occupy a memory region that is already marked as taken.



NotContained

Attempted to operate on a memory region that is not contained within the AllocationTracker.



DoubleFree

Attempted to free memory that has already been deallocated.



Unshrinkable

Fields

required_size: usize

The minimum required size to house currently stored data.

The AllocationTracker does not allow shrinking to the expected size.



Lock(LockingError)

Indicates that a mutex wasn’t lockable.



TrackerInUse

Attempted to borrow the AllocationTracker which is already in use.



Layout(LayoutError)

Tuple Fields

0: LayoutError

The underlying error that caused the Layout to be considered invalid.

Indicates that the provided Layout is invalid.



BorrowMut(RegionBorrowed)

Tried mutably borrowing already borrowed region of memory