Struct contiguous_mem::refs::MemoryReadGuard
source · pub struct MemoryReadGuard<'a, T: ?Sized, Impl: ImplDetails> { /* private fields */ }Expand description
A smart reference wrapper responsible for tracking and managing a flag that indicates whether the memory segment is actively being read from.