pub struct MemoryWriteGuard<'a, T: ?Sized, Impl: ImplDetails> { /* private fields */ }
A smart reference wrapper responsible for tracking and managing a flag that indicates whether the memory segment is actively being written to.