pub struct ImplUnsafe;
A marker struct representing the behavior specialization for unsafe implementation. Should be used when the container is guaranteed to outlive any pointers to data contained in represented memory block.