Struct contiguous_mem::refs::MemoryWriteGuard
source · pub struct MemoryWriteGuard<'a, T: ?Sized, Impl: ImplDetails> { /* private fields */ }Expand description
A smart reference wrapper responsible for tracking and managing a flag that indicates whether the memory segment is actively being written to.