Struct contiguous_mem::ImplDefault
source · pub struct ImplDefault;Expand description
A marker struct representing the behavior specialization that does not require thread-safety. This implementation skips mutexes, making it faster but unsuitable for concurrent usage.